''''
File server of the application
'''
from twisted.spread import pb
from twisted.internet import protocol
from .fileprotocol import FileServerProtocol
from ..helpers.path import avail_filename
from twisted.internet import reactor
from collections import namedtuple
from uuid import uuid4
from ..settings import dest_dir

FileServiceInfo = namedtuple("FileServiceInfo","host port")

class Server():
    '''
    This class represent the main server of the application, really it
    handles and coordinates two servers the control one and the file
    transfer one
    '''
    def __init__(self, uicontr, port):
        
        session_manager = SessionManager() # Global object in common
                                           # that handles the sending
                                           # sessions
        
        # File Server 
        self.file_server = FileServerFactory(uicontr, # To handler monitoring
                                             session_manager,
                                             )
        
        # Spawning File Server
        self._fs_listening = reactor.listenTCP(0, self.file_server)
        self.fs_port = self._fs_listening.getHost().port
        
        # FileReceiverRoot
        fac = pb.PBServerFactory(
            FileReceiverRoot(session_manager, # To handle session
                             uicontr,         # To handle user authorization
                             self.fs_port     # file service port
                             )
            )
        self.factory = fac
        
        # Spawning
        self._listening = reactor.listenTCP(port, fac)
        host = self._listening.getHost()
        self.port = host.port

        
    def stop_listening(self):
        self._listening.stopListening()
        self._fs_listening.stopListening()


class FileServerFactory(protocol.ServerFactory):
    '''
    each connection have to do different things
    
    Ideas: 
    - Since there's a per-connection monitor, maybe I can pass a
    monitorFactory that returns monitors.
    '''
    protocol = FileServerProtocol

    def __init__(self, controller, session_manager):
        self.controller = controller
        self.session_manager=session_manager
    
    def get_file_handler(self,secret):
        '''
        This method asks the session_manager to get info about the
        session and returns a file_handler appropriate for saving data
        into the file
        '''
        data = self.session_manager.get(secret)
        fh = data["file_handler"]
        return fh
        
    
    def get_monitor(self, secret):
        '''
        Same as file_handler
        '''
        data = self.session_manager.get(secret)
        return data["monitor"]

    def check(self, secret):
        return self.session_manager.check(secret)

class SessionManager(object):
    '''
    This manages the sessions opened with clients
    '''
    def __init__(self):
        '''
        '''
        self._store = {}

    def register(self,secret,info):
        '''
        '''
        self._store[secret] = info

    def check(self, secret):
        '''
        '''
        data = self._store.get(secret)
        if data:
            return True
        else:
            return False
    
    def get(self, secret):
        return self._store.get(secret)

    def unregister(self, secret):
        '''
        '''
        self._store.pop(secret)
        

class FileReceiverRoot(pb.Root):
    '''
    This class takes as it first argument a controller
    '''

    def __init__(self, session_manager, contr, port):
        self.controller=contr # ui
        self.session_manager = session_manager
        self.file_service=port
    
    def remote_request_session(self, name, basename, size):
        '''    
        Authorize a client to send some data under basename, and with
        size
        This method
        '''
        # Perform external authorization
        res = self.controller.authorize(name, basename, size)
        
        if res:
            secret = str(uuid4())
            # Perform registering in some way that this is added
            # Feeds the session data with all the stuff useful
            monitor = self.controller.get_monitor(name, basename, size)
            file_handler = self.controller.get_file_handler(basename, size)
                        
            
            data = dict(name=name, 
                        basename=basename,
                        size=size,
                        monitor=monitor,
                        file_handler=file_handler)
            
            self.session_manager.register(secret, data)
            return secret
        else:
            return None
        
    def remote_get_folder_maker(self, secret):
        if self.session_manager.check(secret):
            fm =  FolderMaker()
            return fm

    def remote_get_directives(self):
        '''
        Returns the info abot the file service spawned on the server
        machine.
        '''
        return self.file_service 

import os, logging

class FolderMaker(pb.Referenceable):

    def remote_mkdir(self, directory):
        dest = os.path.join(dest_dir(),directory)
        # For now I can't implement the not-overriding functions,
        # unfortunately
        logging.debug("Making directory %s"%dest)
        os.mkdir(dest)
