#!/usr/bin/python
'''
Module used to define a good interface for zeroconf service discovery.
For now it's based on avahi/dbus

It's based on snippets/avahi/browse.py

The classes defined here are implementation of abstract classes
defined in module generic.py

Avahi Hacking Docs
------------------

These are additional docs that are absent on avahi documentation, just
to understand how avahi/dbus harness works.

.. class ServiceBrowser::

   This is the class that discovers the services, once we've
   discovered them we have to resolve their address or so with the
   :class:`Server` class

.. event ItemNew::
   
   A new service is discovered

.. event ItemRemove::

   A service has disappeared

.. class Server::

   This is the Avahi Server class that we access throught dbus.

.. function server.ResolveService(interface, protocol, name,
                                  type, domain, avahi.PROTO_INET,
                                  dbus.UInt32(0), reply_handler=service_resolved_cb,
                                  error_handler=error_cb)::

    When a new service is added, you can use resolveservice to obtain
    address port and other niceties

.. class EntryGroup::

   Something related with the publishing of a service

.. event StateChanged::

.. callback state_changed_callback(state, error)

    a state could be something like that (fetched from an avahi source
    file ga-entry-group.h)
    
    :param state:
    
    enum GaEntryGroupState

    Enumerator:
    GA_ENTRY_GROUP_STATE_UNCOMMITED 	
    GA_ENTRY_GROUP_STATE_REGISTERING 	
    GA_ENTRY_GROUP_STATE_ESTABLISHED 	
    GA_ENTRY_GROUP_STATE_COLLISTION 	
    GA_ENTRY_GROUP_STATE_FAILURE 

    :param error:
    
    http://library.gnome.org/devel/gconf/stable/gconf-gconf-error.html

'''
import avahi
import dbus
import gtk
import avahi.ServiceTypeDatabase

# Helper functions to clean a bit the code, the dbus code is ugly
from helpers import (get_avahi_server,
                     get_avahi_entry_group,
                     get_avahi_service_browser)

# Data structures used to pass parameters
from generic import ServiceInfo, ServiceResolvedInfo

# Interfaces
from generic import ServiceBrowser,ServicePublisher

import logging


try:
    import dbus.glib
except ImportError, e:
    pass

class EntryGroupState: # Should be connection group state
    '''
    This is simply an enum to identify the status of the published connection
    '''
    UNCOMMITTED,REGISTERING,ESTABILISHED,COLLISION,FAILURE = range(5)

class ServiceTypeDatabase:
    '''
    Used to have a nice representation of services
    '''
    def __init__(self):
        self.pretty_name = avahi.ServiceTypeDatabase.ServiceTypeDatabase()

    def get_human_type(self, type):
	if self.pretty_name.has_key(type):
	    return self.pretty_name[type]
        else:
            return type


class AvahiServiceDiscovery(ServiceBrowser):
    '''
    Main class for interacting with avahi services throught dbus
    
    Attributes:
    - `server` is an instance of the avahi Server class
    - `type` service type that we're looking for
    '''
    
    def __init__(self,type):
        #Start Service Discovery
	self.domain = ""
        self.type = type
        
        self.system_bus = dbus.SystemBus()

	self.service_browsers = {}
        
	self.start_service_discovery()
        
    # Public signals, template method definition

    def on_service_found(self, service_res_info):
        return

    def on_service_removed(self, service_info):
        return

    def on_error(self, err):
        return
    
    def on_discovery_finished(self):
        return
    
    # Public functions, that can be used externally
    def start_service_discovery(self, component=None, verb=None, applet=None):
	if len(self.domain) != 0:
            logging.debug( "domain not null %s" % (self.domain))
            logging.debug("Already Discovering")
	    return 

        self.server = get_avahi_server()
        self.domain = self.server.GetDomainName()

	try: 
		self.use_host_names = self.server.IsNSSSupportAvailable()
	except:
		self.use_host_names = False  

        logging.debug( "Starting discovery")

	self.interface = avahi.IF_UNSPEC
	self.protocol = avahi.PROTO_INET

        self.add_service_type(self.interface, self.protocol, self.type, self.domain)
        
    def stop_service_discovery(self, component=None, verb=None, applet=None):
	if len(self.domain) == 0:
            logging.debug("Discovery already stopped")
	    return

        logging.debug("Discovery stopped")

    
    # Private/implementation related functions
    
    def siocgifname(self, interface):
        '''
        Not idea what's this
        '''
	if interface <= 0:
	    return "any"
	else:
	    return self.server.GetNetworkInterfaceNameByIndex(interface)

    def _on_service_resolved(self, interface, protocol,
                         name, type, domain, host,
                         aprotocol, address,
                         port, txt, flags):
        '''
        Important it resove all the stuff that we need for connecting
        to the discovered services
        '''
        # Passing "massaged" data to the custom callback
        self.on_service_found(ServiceResolvedInfo(interface=interface,
                                                  protocol=str(protocol),
                                                  name=str(name),
                                                  type=str(type),
                                                  domain=str(domain),
                                                  host=str(host),
                                                  address=str(address),
                                                  port=port,
                                                  txt=str(txt))
                              )

    def _on_service_new(self, interface, protocol, name, type, domain, flags):
        # this check is for local services
        try:
            if flags & avahi.LOOKUP_RESULT_LOCAL:
                logging.debug("This is a local service")
        except dbus.DBusException:
            pass

        self.server.ResolveService(interface,
                                   protocol,
                                   name, type,
                                   domain, avahi.PROTO_INET,
                                   dbus.UInt32(0), 
                                   reply_handler=self._on_service_resolved,
                                   error_handler=self.on_error)

    def _on_service_remove(self, interface, protocol, name, type, domain, flags):
        self.on_service_removed(ServiceInfo(interface=interface,
                                            protocol=str(protocol),
                                            name=str(name),
                                            type=str(type),
                                            domain=str(domain)))

    def add_service_type(self, interface, protocol, type, domain):
	# Are we already browsing this domain for this type? 
	if self.service_browsers.has_key((interface, protocol, type, domain)):
	    return

        b = get_avahi_service_browser(self.server, interface,
                                      protocol,
                                      type,
                                      domain)

        b.connect_to_signal('ItemNew', self._on_service_new)
        b.connect_to_signal('ItemRemove', self._on_service_remove)
        b.connect_to_signal('Failure', self.on_error)
        b.connect_to_signal('AllForNow', self.on_discovery_finished)
	
	self.service_browsers[(interface, protocol, type, domain)] = b

    
class AvahiServicePublisher(ServicePublisher):
    '''
    This class is used to publish a service on avahi.
    '''
    def __init__(self,service_name, service_type, port="8080",
                 domain="", host="", service_txt = "somethingcrazy"):
        '''
        :param service_name: A name to recognize the service
        :param service_type: The type, it usually is something like _hello._tcp

        :param port: which port the service is located
        :param domain:
        :param host:
        :param service_txt:
        '''
        # Initializating dbus stuff
        server = get_avahi_server()
        group = get_avahi_entry_group(server)
        
        # Connecting important stuff
        group.connect_to_signal(
            "StateChanged", 
            self._on_state_changed)
        
        # Don't know what is this stuff, this should be avahi api plus
        # dbus wizardry
        group.AddService(
            avahi.IF_UNSPEC,
            avahi.PROTO_UNSPEC,
            dbus.UInt32(0),
            service_name,
            service_type,
            domain,host,
            dbus.UInt16(port),
            avahi.string_array_to_txt_array(service_txt))
        group.Commit()
        self.group = group
        
    # Public methods, to override in subclasses
    
    def on_connect(self):pass

    def on_error(self):pass

    # Private stuff, it handles raw dbus signals
    def _on_state_changed(self, state, error):
        '''
        Method use to dispatch various function on state changing.
        '''
        if state == EntryGroupState.ESTABILISHED:
            self.on_connect()
        elif state == EntryGroupState.FAILURE:
            self.on_error(error)
    
    def __del__(self):
        # If there are no errors
        if hasattr(self,"group"):
            self.group.Free()

def main():
    logging.basicConfig(level=logging.DEBUG)
    AvahiServiceDiscovery("_hello._tcp")
    gtk.main()

if __name__ == "__main__":
    main()
